//============================================================================================//
//                                                                                            //
//                                          C  Client                                         //
//                                          ---------                                         //
//                                                                                            //
//  This is an example of a client application in C for FluidProp, a server module            //
//  for the calculation of thermophysical properties of fluids. FluidProp is a common         //
//  interface to GasMix, IF97, RefProp, (free)StanMix, TPSI, PCP-SAFT and is developed        //
//  by Piero Colonna and Teus van der Stelt.                                                  //
//                                                                                            //
//  Teus van der Stelt                                                                        //
//  Asimptote BV                                                                              //
//                                                                                            //
//  January, 2007, for FluidProp 2                                                            //
//  April, 2007, for FluidProp 2.3                                                            //
//  May, 2009, for FluidProp 2.3                                                              //
//  November, 2018, for FluidProp 3.1 for macOS and Linux                                     //
//                                                                                            //
//============================================================================================//

#include <stdio.h>
#include <string.h>

#include "fluidprop.h"

#define LEN_FLUIDNAMES 256
#define LEN_COMPONENTS 32

void PressEnter()
{
   printf( "Press Enter to continue...\n");
   getchar();
}

void PrintFluids( char names[][LEN_FLUIDNAMES], int count, int width, int columns )
{
   int i;
   printf( "Number of available fluids = %d\n" , count );
   printf( "The available fluids are: \n" );

   for( i = 0; i < count; i++ )
   {
      if ( i > 0 && i % columns == 0)
         printf( "\n" );

      printf( "%-*s", width, names[i] );
   }
}

int main(int argc, char* argv[])
{
   int  version[4];

   // Number of components is generally not limited, but RefProp does not allow more
   // than 20 components in a mixture and (free)StanMix does not allow more than 10. 
   // It is important however to allocate sufficient memory space.
   char     Comp[20][LEN_COMPONENTS];
   double   Conc[20];

   int nFluids;
   char FluidNames[1000][LEN_FLUIDNAMES];

   BOOL TPSI_Available, freeStanMix_Available, IF97_Available, GasMix_Available, RefProp_Available, 
        StanMix_Available, PCP_SAFT_Available;

   double P, T, h, s;
   double x[8], y[8];
   struct fluidstate_t state;

   // Before you can use FluidProp you must create and initialize a FluidProp object 
   // with the init_fluidProp function.
   init_fluidprop();

   // Before a property can be calculated, first a child object for the desired thermophysical 
   // library must be created. This can be done by using the fluidprop_createobject function 
   // or by defining the working fluid directly with the fluidprop_setfluid function. However
   // function fluidprop_setfluid can be used for the same purpose too, with the function
   // fluidprop_createobject you can easily create a list of available libraries in 
   // your client.
   fluidprop_getversion("FluidProp", version);
 
   printf( "\n" );
   printf( "Available libraries in FluidProp version %d.%d.%d.%d\n",version[0],version[1],version[2],version[3]);
   printf( "----------------------------------------------------------------------------\n" );

   // Create an object for the TPSI library
   fluidprop_createobject( "TPSI");
   if ( !strcmp(fluidprop_geterror(), "No errors") )
   {
      fluidprop_getversion("TPSI", version);
      printf( "TPSI version          %d.%d.%d.%d available\n", version[0], version[1], version[2], version[3] );
      TPSI_Available = TRUE;
   }
   else
   {
      printf( "Unable to create TPSI object       : %s\n", fluidprop_geterror());
      TPSI_Available = FALSE;
   }

   // Create an object for the freeStanMix library
   fluidprop_createobject( "freeStanMix");
   if ( !strcmp(fluidprop_geterror(), "No errors") )
   {
      fluidprop_getversion("freeStanMix", version);
      printf( "freeStanMix version   %d.%d.%d.%d available\n", version[0], version[1], version[2], version[3] );
      freeStanMix_Available = TRUE;
   }
   else
   {
      printf( "Unable to create freeStanMix object: %s\n", fluidprop_geterror());
      freeStanMix_Available = FALSE;
   }

   // Create an object for the IF97 library
   fluidprop_createobject( "IF97");
   if ( !strcmp(fluidprop_geterror(), "No errors") )
   {
      fluidprop_getversion("IF97", version);
      printf( "IF97 version          %d.%d.%d.%d available\n", version[0], version[1], version[2], version[3] );
      IF97_Available = TRUE;
   }
   else
   {
      printf( "Unable to create IF97 object       : %s\n", fluidprop_geterror());
      IF97_Available = FALSE;
   }

   // Create an object for the GasMix library
   fluidprop_createobject( "GasMix");
   if ( !strcmp(fluidprop_geterror(), "No errors") )
   {
      fluidprop_getversion("GasMix", version);
      printf( "GasMix version        %d.%d.%d.%d available\n", version[0], version[1], version[2], version[3] );
      GasMix_Available = TRUE;
   }
   else
   {
      printf( "Unable to create GasMix object     : %s\n", fluidprop_geterror());
      GasMix_Available = FALSE;
   }

   // Create an object for the RefProp library (only available in the Full version of FluidProp)
   // Note that an additional Add-on is required in order to run RefProp or, that, 
   // if you are using MS-Windows, a version of REFPROP obtained from NIST has been 
   // installed. A license for the Add-on can be purchased from the Asimptote web shop. 
   // If the appropriate license is not available on your system then an error message 
   // is displayed.
   fluidprop_createobject( "RefProp");
   if ( !strcmp(fluidprop_geterror(), "No errors") )
   {
      fluidprop_getversion("RefProp", version);
      printf( "RefProp version       %d.%d.%d.%d available\n", version[0], version[1], version[2], version[3] );
      RefProp_Available = TRUE;
   }
   else
   {
      printf( "Unable to create RefProp object    : %s\n", fluidprop_geterror());
      RefProp_Available = FALSE;
   }

   // Create an object for the StanMix library
   // Note that an additional Add-on is required in order to run StanMix. A license for this 
   // Add-on can be purchased from the Asimptote web shop. If the appropriate license is not 
   // available on your system then an error message is displayed.
   fluidprop_createobject( "StanMix");
   if ( !strcmp(fluidprop_geterror(), "No errors") )
   {
      fluidprop_getversion("StanMix", version);
      printf( "StanMix version       %d.%d.%d.%d available\n", version[0], version[1], version[2], version[3] );
      StanMix_Available = TRUE;
   }
   else
   {
      printf( "Unable to create StanMix object    : %s\n", fluidprop_geterror());
      StanMix_Available = FALSE;
   }

   // Create an object for the PCP-SAFT library
   // Note that an additional Add-on is required in order to run PCP-SAFT. A license for this 
   // Add-on can be purchased from the Asimptote web shop. If the appropriate license is not 
   // available on your system then an error message is displayed.
   fluidprop_createobject( "PCP-SAFT");
   if ( !strcmp(fluidprop_geterror(), "No errors") )
   {
      fluidprop_getversion("PCP-SAFT", version);
      printf( "PCP-SAFT version      %d.%d.%d.%d available\n", version[0], version[1], version[2], version[3] );
      PCP_SAFT_Available = TRUE;
   }
   else
   {
      printf( "Unable to create PCP-SAFT object   : %s\n", fluidprop_geterror());
      PCP_SAFT_Available = FALSE;
   }

   printf( "\n" );
   PressEnter();

   //----------------------------------------------------------------------------------//
   //                                       TPSI                                       //
   //----------------------------------------------------------------------------------//

   if ( TPSI_Available)
   {
      // After initialization of the library, first the working fluid
      // must be defined before a property can be calculated. In this example methane
      // from the TPSI library is defined as the working fluid. For TPSI there is no
      // need to specify concentation Conc, because there is always just 1 component.

      strcpy (Comp[0], "Methane");
      fluidprop_setfluid( "TPSI", 1, Comp[0], LEN_COMPONENTS, Conc );

      if ( strcmp(fluidprop_geterror(), "No errors") )
      {
         printf( "error: \n" );
         printf("%s\n", fluidprop_geterror());
      }
      else
      {
         printf( "\n" );
         printf( "TPSI, Methane, P = 1 bar, T = 15 degr.C\n" );
         printf( "----------------------------------------------------------------------------\n" );

         P = 1;
         T = 15;

         // To calculate some properties as function of two other independent properties
         // the appropriate function must be called. In the example below the density,
         // specific volume, enthalpy, entropy and vapor quality are calculated as
         // function of the pressure and temperature. There is a check if the pressure
         // and temperature function return the same value as entered.

         printf( "Pressure      = %e%s\n" , fluidprop_pressure   ( "PT", P, T) , " bar    " );
         if ( strcmp(fluidprop_geterror(), "No errors") ) printf("%s\n", fluidprop_geterror());
         printf( "Temperature   = %e%s\n" , fluidprop_temperature( "PT", P, T) , " degr. C" );
         if ( strcmp(fluidprop_geterror(), "No errors") ) printf("%s\n", fluidprop_geterror());
         printf( "Density       = %e%s\n" , fluidprop_density    ( "PT", P, T) , " kg/m3  " );
         if ( strcmp(fluidprop_geterror(), "No errors") ) printf("%s\n", fluidprop_geterror());
         printf( "Spec. volume  = %e%s\n" , fluidprop_specvolume ( "PT", P, T) , " m3/kg  " );
         if ( strcmp(fluidprop_geterror(), "No errors") ) printf("%s\n", fluidprop_geterror());
         printf( "Enthalpy      = %e%s\n" , fluidprop_enthalpy   ( "PT", P, T) , " kJ/kg  " );
         if ( strcmp(fluidprop_geterror(), "No errors") ) printf("%s\n", fluidprop_geterror());
         printf( "Entropy       = %e%s\n" , fluidprop_entropy    ( "PT", P, T) , " kJ/kg.K" );
         if ( strcmp(fluidprop_geterror(), "No errors") ) printf("%s\n", fluidprop_geterror());
         printf( "Vapor quality = %e%s\n" , fluidprop_vaporqual  ( "PT", P, T) , " kg/kg  " );
         if ( strcmp(fluidprop_geterror(), "No errors") ) printf("%s\n", fluidprop_geterror());
         printf( "\n" );

         // You can retrieve a list of available fluids in a library by calling
         // functions getfluidnames. The number of fluids displayed depends on the 
         // license and not all fluids are available for each sub-library. For the 
         // free version the number of fluids is most limited. 

         fluidprop_getfluidnames( "long", "TPSI", &nFluids, FluidNames[0], LEN_FLUIDNAMES);

         printf( "\n" );
         printf( "Availables fluids in TPSI\n" );
         printf( "----------------------------------------------------------------------------\n" );

         if ( !strcmp(fluidprop_geterror(), "No errors"))
            PrintFluids( FluidNames, nFluids, 22, 3 );
      }

      printf( "\n" );
      PressEnter();

   }

   // For the other libraries the method is essentially the same. The only
   // difference is in the way arguments are supplied to the setfluid function.
   // Furthermore the properties that can be calculated varies for each library,
   // as well as the two the independent properties that can be used as input
   // (see user documentation).
   // In the examples below for each library the use of subroutine setfluid is
   // explained in detail, together with the calculation of several properties.

   //----------------------------------------------------------------------------------//
   //                                     freeStanMix                                  //
   //----------------------------------------------------------------------------------//

   if ( freeStanMix_Available)
   {
      // You can retrieve a list of available fluids in a library by calling
      // function getfluidnames. The number of fluids and an array with fluidnames
      // are returned.

      fluidprop_getfluidnames( "long", "freeStanMix", &nFluids, FluidNames[0], LEN_FLUIDNAMES);

      printf( "\n" );
      printf( "Available fluids in freeStanMix\n" );
      printf( "----------------------------------------------------------------------------\n" );

      if ( !strcmp(fluidprop_geterror(), "No errors")) 
         PrintFluids( FluidNames, nFluids, 22, 3 );

      printf( "\n" );
      PressEnter();

      // For freeStanMix you have three possiblities: you can use pure fluids, predefined mixtures,
      // or you can use mixtures and assign different mole fractions to concentrations of
      // of the components.

      // In the first example a pure fluid is selected. There is no need to specify a
      // concentration, it is sufficient to specify the fluidname, in this case ethanol,
      // and the number of components being 1.

      strcpy (Comp[0], "ethanol");
      fluidprop_setfluid( "freeStanMix", 1, Comp[0], LEN_COMPONENTS, Conc);

      if ( strcmp(fluidprop_geterror(), "No errors") )
      {
         printf( "\n" );
         printf("%s\n", fluidprop_geterror());
      }
      else
      {
         printf( "\n" );
         printf( "freeStanMix, ethanol, P = 2 bar, T = 120 degr.C\n" );
         printf( "----------------------------------------------------------------------------\n" );

         P = 2;
         T = 120;

         // To calculate some properties as function of two other independent properties
         // the appropriate function must be called. In the example a large number of
         // properties is calculated. Next to various thermodynamic properties also
         // properties related to CFD quantities are available. For clarity the writing
         // of the error message has been left out.

         printf( "Pressure      = %e%s\n" , fluidprop_pressure   ( "PT", P, T) , " bar      " );
         printf( "Temperature   = %e%s\n" , fluidprop_temperature( "PT", P, T) , " degr. C  " );
         printf( "Density       = %e%s\n" , fluidprop_density    ( "PT", P, T) , " kg/m3    " );
         printf( "Spec. volume  = %e%s\n" , fluidprop_specvolume ( "PT", P, T) , " m3/kg    " );
         printf( "Enthalpy      = %e%s\n" , fluidprop_enthalpy   ( "PT", P, T) , " kJ/kg    " );
         printf( "Entropy       = %e%s\n" , fluidprop_entropy    ( "PT", P, T) , " kJ/kg.K  " );
         printf( "Vapor quality = %e%s\n" , fluidprop_vaporqual  ( "PT", P, T) , " kg/kg    " );
         printf( "Int. energy   = %e%s\n" , fluidprop_intenergy  ( "PT", P, T) , " kJ/kg    " );
         printf( "Isoc. Heat Cv = %e%s\n" , fluidprop_heatcapv   ( "PT", P, T) , " kJ/kg.K  " );
         printf( "Isob. Heat Cp = %e%s\n" , fluidprop_heatcapp   ( "PT", P, T) , " kJ/kg.K  " );
         printf( "Sound speed   = %e%s\n" , fluidprop_soundspeed ( "PT", P, T) , " m/s      " );


         // Note that an additional Add-on is required in order to compute the derivatives. 
         // A license for this Add-on can be purchased from the Asimptote web shop. If the 
         // appropriate license is not available on your system then an error message is 
         // displayed.             
         fluidprop_alpha( "PT", P, T);
         
         if ( strcmp(fluidprop_geterror(), "No errors") ) 
            printf( "\n%s\n", fluidprop_geterror());
         else 
         {
            printf( "Alpha         = %e%s\n" , fluidprop_alpha      ( "PT", P, T) , " m2/s2    " );
            printf( "Beta          = %e%s\n" , fluidprop_beta       ( "PT", P, T) , " kg/m3    " );
            printf( "Chi           = %e%s\n" , fluidprop_chi        ( "PT", P, T) , " Nm4/kg2.K" );
            printf( "Fi            = %e%s\n" , fluidprop_fi         ( "PT", P, T) , " 1/K      " );
            printf( "Ksi           = %e%s\n" , fluidprop_ksi        ( "PT", P, T) , " kg2/Nm4  " );
            printf( "Psi           = %e%s\n" , fluidprop_psi        ( "PT", P, T) , " s2/m2    " );
            printf( "Zeta          = %e%s\n" , fluidprop_zeta       ( "PT", P, T) , " kg.K/m3  " );
            printf( "Theta         = %e%s\n" , fluidprop_theta      ( "PT", P, T) , " 1/K      " );
            printf( "Kappa         = %e%s\n" , fluidprop_kappa      ( "PT", P, T) , " 1/bar    " );
            printf( "Gamma         = %e%s\n" , fluidprop_gamma      ( "PT", P, T) , " -        " );
         }
      }

      printf( "\n" );
      PressEnter();

      // In the second example a predefined mixture is selected. Here is also no need
      // to specify a concentration, because it has been stored in the mixture input file
      // file. It is sufficient to specify the mixture name, in this case propane/pentane
      // (mixture of 50% mole percent propane and 50% mole percent pentane). For the use
      // of the mixture with the stored concentrations, the number of components must be 1.

      strcpy (Comp[0], "propane/pentane");
      fluidprop_setfluid( "freeStanMix", 1, Comp[0], LEN_COMPONENTS, Conc);

      if ( strcmp(fluidprop_geterror(), "No errors") )
         printf( "\n%s\n", fluidprop_geterror());
      else
      {
         printf( "\n" );
         printf( "freeStanMix, propane/pentane (predef. mix: 50%%-50%%), P = 5 bar, T = 50 degr.C\n" );
         printf( "-----------------------------------------------------------------------------\n" );

         P = 5;
         T = 50;

         // To calculate some properties as function of two other independent properties
         // the appropriate function must be called. In this example additionally the
         // liquid and vapor phase composition are calculated. For clarity the writing
         // of the error message has been left out.

         printf( "Pressure      = %e%s\n" , fluidprop_pressure   ( "PT", P, T) , "  bar    " );
         printf( "Temperature   = %e%s\n" , fluidprop_temperature( "PT", P, T) , "  degr.C " );
         printf( "Spec. volume  = %e%s\n" , fluidprop_specvolume ( "PT", P, T) , "  m3/kg  " );
         printf( "Density       = %e%s\n" , fluidprop_density    ( "PT", P, T) , "  kg/m3  " );
         printf( "Enthalpy      = %e%s\n" , fluidprop_enthalpy   ( "PT", P, T) , "  kJ/kg  " );
         printf( "Entropy       = %e%s\n" , fluidprop_entropy    ( "PT", P, T) , "  kJ/kg.K" );
         printf( "Vapor qual.   = %e%s\n" , fluidprop_vaporqual  ( "PT", P, T) , "  kg/kg  " );
         printf( "\n" );

         fluidprop_liquidcmp("PT", P, T, x);
         printf( "Liq phase cmp = propane : %e%s\n" , x[0] , " mole frac " );
         printf( "                pentane : %e%s\n" , x[1] , " mole frac " );
         printf( "\n" );

         fluidprop_vaporcmp ("PT", P, T, y);
         printf( "Vap phase cmp = propane : %e%s\n" , y[0] , " mole frac " );
         printf( "                pentane : %e%s\n" , y[1] , " mole frac " );
         printf( "\n" );
      }

      printf( "\n" );
      PressEnter();

      // In the third freeStanMix example the same mixture is selected as in the second
      // example, but now the mole fractions are adapted. To do this you must specify
      // the real number of components in the mixture, the mixture name and concentrations.
      // In most of the available mixtures the first component comes first in the mixture
      // name and its concentration must be stored in the first element of the concentration
      // array; the concentration of the second component must be stored in the second
      // element, etc. So, concentrations must be stored in the order as the component names
      // appear in the mixture name.

      strcpy (Comp[0], "propane/pentane");
      Conc[0] = 0.75;
      Conc[1] = 1 - Conc[0];
      fluidprop_setfluid( "freeStanMix", 2, Comp[0], LEN_COMPONENTS, Conc);

      if ( strcmp(fluidprop_geterror(), "No errors") )
         printf( "\n%s\n", fluidprop_geterror());
      else
      {
         printf( "\n" );
         printf( "freeStanMix, propane/pentane (predef. mix: 75%%-25%%), P = 5 bar, T = 50 degr.C\n" );
         printf( "----------------------------------------------------------------------------\n" );

         P = 5;
         T = 50;

         // To calculate some properties as function of two other independent properties
         // the appropriate function must be called. In this example the same properties
         // are calculated as in the example above.

         printf( "Pressure      = %e%s\n" , fluidprop_pressure   ( "PT", P, T) , "  bar    " );
         printf( "Temperature   = %e%s\n" , fluidprop_temperature( "PT", P, T) , "  degr.C " );
         printf( "Spec. volume  = %e%s\n" , fluidprop_specvolume ( "PT", P, T) , "  m3/kg  " );
         printf( "Density       = %e%s\n" , fluidprop_density    ( "PT", P, T) , "  kg/m3  " );
         printf( "Enthalpy      = %e%s\n" , fluidprop_enthalpy   ( "PT", P, T) , "  kJ/kg  " );
         printf( "Entropy       = %e%s\n" , fluidprop_entropy    ( "PT", P, T) , "  kJ/kg.K" );
         printf( "Vapor qual.   = %e%s\n" , fluidprop_vaporqual  ( "PT", P, T) , "  kg/kg  " );
         printf( "\n" );

         fluidprop_liquidcmp("PT", P, T, x);
         printf( "Liq phase cmp = Benzene : %e%s\n" , x[0] , " mole frac " );
         printf( "                Toluene : %e%s\n" , x[1] , " mole frac " );
         printf( "\n" );

         fluidprop_vaporcmp ("PT", P, T, y);
         printf( "Vap phase cmp = Benzene : %e%s\n" , y[0] , " mole frac " );
         printf( "                Toluene : %e%s\n" , y[1] , " mole frac " );
         printf( "\n" );
      }

      printf( "\n" );
      PressEnter();
   }

   //----------------------------------------------------------------------------------//
   //                               IF97 water and steam                               //
   //----------------------------------------------------------------------------------//

   if ( IF97_Available)
   {
      int i;

      // IF97 just contains a model for water and steam, so for IF97 the number of
      // components, component name(s) and concentrations do not matter. It is
      // sufficient to specify the library name "IF97" and to declare the arrays
      // Comp and Conc as a character array and a double array in the way it is
      // done in the top of this sample program. So, there is also no need to retrieve
      // a list of available fluids by calling function getfluidnames.

      // Re-init concentrations and component names. This is not really necessary but
      // it shows the independence for IF97.
      for( i = 0; i < 20; i++)
      {
         Conc[i] = 0;
         strcpy (Comp[i], " ");
      }
      fluidprop_setfluid( "IF97", 1, Comp[0], LEN_COMPONENTS, Conc);

      if ( strcmp(fluidprop_geterror(), "No errors") )
         printf( "\n%s\n", fluidprop_geterror());
      else
      {
         printf( "\n" );
         printf( "IF97 water/steam, P = 40 bar, h = 3000 kJ/kg\n" );
         printf( "----------------------------------------------------------------------------\n" );

         P = 40;
         h = 3000;

         // To calculate some properties as function of two other independent properties
         // the appropriate function must be called, exactly like in other examples. In
         // this example all available functions for IF97 are called and for the input
         // the pressure and enthalpy are entered. For clarity the writing of the error
         // message has been left out.

         printf( "Pressure      = %e%s\n" , fluidprop_pressure   ( "Ph", P, h) , " bar    " );
         printf( "Temperature   = %e%s\n" , fluidprop_temperature( "Ph", P, h) , " degr.C " );
         printf( "Spec. volume  = %e%s\n" , fluidprop_specvolume ( "Ph", P, h) , " m3/kg  " );
         printf( "Density       = %e%s\n" , fluidprop_density    ( "Ph", P, h) , " kg/m3  " );
         printf( "Enthalpy      = %e%s\n" , fluidprop_enthalpy   ( "Ph", P, h) , " kJ/kg  " );
         printf( "Entropy       = %e%s\n" , fluidprop_entropy    ( "Ph", P, h) , " kJ/kg.K" );
         printf( "Int. Energy   = %e%s\n" , fluidprop_intenergy  ( "Ph", P, h) , " kJ/kg  " );
         printf( "Vapor qual.   = %e%s\n" , fluidprop_vaporqual  ( "Ph", P, h) , " kg/kg  " );
         printf( "Isoc. Heat Cv = %e%s\n" , fluidprop_heatcapv   ( "Ph", P, h) , " kJ/kg.K" );
         printf( "Isob. Heat Cp = %e%s\n" , fluidprop_heatcapp   ( "Ph", P, h) , " kJ/kg.K" );
         printf( "Sound speed   = %e%s\n" , fluidprop_soundspeed ( "Ph", P, h) , " m/s    " );
         printf( "Dyn. Viscos.  = %e%s\n" , fluidprop_viscosity  ( "Ph", P, h) , " Pa.s   " );
         printf( "Th. Conduct.  = %e%s\n" , fluidprop_thermcond  ( "Ph", P, h) , " W/m.K  " );
      }

      printf( "\n" );
      PressEnter();
   }

   //----------------------------------------------------------------------------------//
   //                                     GasMix                                       //
   //----------------------------------------------------------------------------------//

   if ( GasMix_Available)
   {
      // Using GasMix you can calculate properties of ideal gases and mixtures of ideal
      // gasses. You can retrieve a list of available fluids by calling function
      // getfluidnames. The number of fluids and an array with fluidnames are returned.

      fluidprop_getfluidnames( "short", "GasMix", &nFluids, FluidNames[0], LEN_FLUIDNAMES);

      printf( "\n" );
      printf( "Available fluids in GasMix\n" );
      printf( "----------------------------------------------------------------------------\n" );

      if ( !strcmp(fluidprop_geterror(), "No errors"))
         PrintFluids( FluidNames, nFluids, 22, 3);

      printf( "\n" );
      PressEnter();

      // For GasMix there is just one possibility to define the working fluid. You must 
      // specify the composition by specifying successively the number of components, the
      // components and the molefractions. At the moment there is no feature for predefined 
      // mixtures.

      // In the example below an air compostion is specified to be the working fluid. The 
      // order of the mole fractions in array Conc must be according the order of the
      // component names in array Comp. It is also possible to define a pure fluid by
      // specifying one component with a mole fraction equaling 1.

      strcpy (Comp[0], "N2"   );  Conc[0] = 0.7801;
      strcpy (Comp[1], "O2"   );  Conc[1] = 0.2094;
      strcpy (Comp[2], "H2O"  );  Conc[2] = 0.0102;
      strcpy (Comp[3], "CO2"  );  Conc[3] = 0.0003;
      fluidprop_setfluid( "GasMix", 4, Comp[0], LEN_COMPONENTS, Conc);

      if ( strcmp(fluidprop_geterror(), "No errors") )
         printf( "\n%s\n", fluidprop_geterror());
      else
      {
         printf( "\n" );
         printf( "GasMix, Air, P = 1 bar, T = 15 degr.C\n" );
         printf( "----------------------------------------------------------------------------\n" );

         P = 1;
         T = 15;

         // To calculate some properties as function of two other independent properties
         // the appropriate function must be called, exactly like in other examples. For 
         // clarity the writing of the error message has been left out.

         h = fluidprop_enthalpy( "PT", P, T);
         s = fluidprop_entropy ( "Ph", P, h);
         printf( "Enthalpy      = %e%s\n" , h                                            , " kJ/kg  " );
         printf( "Entropy       = %e%s\n" , s                                            , " kJ/kg.K" );
         printf( "Pressure      = %e%s\n" , fluidprop_pressure   ( "Ps", P, s) , " bar    " );
         printf( "Temperature   = %e%s\n" , fluidprop_temperature( "Ph", P, h) , " degr.C " );
         printf( "Spec. volume  = %e%s\n" , fluidprop_specvolume ( "Ps", P, s) , " m3/kg  " );
         printf( "Density       = %e%s\n" , fluidprop_density    ( "Ph", P, h) , " kg/m3  " );
         printf( "Isob. Heat Cp = %e%s\n" , fluidprop_heatcapp   ( "PT", P, T) , " kJ/kg.K" );
         printf( "Dyn. Viscos.  = %e%s\n" , fluidprop_viscosity  ( "PT", P, T) , " Pa.s   " );
         printf( "Th. Conduct.  = %e%s\n" , fluidprop_thermcond  ( "PT", P, T) , " W/m.K  " );
      }

      printf( "\n" );
      PressEnter();
   }

   //----------------------------------------------------------------------------------//
   //                                     RefProp                                      //
   //----------------------------------------------------------------------------------//

   if ( RefProp_Available)
   {
      // Using RefProp you can calculate properties of pure fluids and mixtures of
      // fluids. You can retrieve a list of available fluids by calling function
      // getfluidnames. The number of fluids and an array with fluidnames are returned.

      // Note that an additional Add-on is required in order to run RefProp or, that, 
      // if you are using MS-Windows, a version of REFPROP obtained from NIST has been 
      // installed. A license for the Add-on can be purchased from the Asimptote web shop. 
      // If the appropriate license is not available on your system then an error message 
      // is displayed.

      fluidprop_getfluidnames( "l", "RefProp", &nFluids, FluidNames[0], LEN_FLUIDNAMES);

      printf( "\n" );
      printf( "Available fluids in RefProp\n" );
      printf( "----------------------------------------------------------------------------\n" );

      if ( !strcmp(fluidprop_geterror(), "No errors"))
         PrintFluids( FluidNames, nFluids, 22, 3 );

      printf( "\n" );
      PressEnter();

      // In RefProp there are two ways to define the working fluid. First you can specify
      // the composition by specifying successively the number of components, the component
      // names and the mole fractions, like in GasMix. Second you can define predefined
      // mixtures that are available in RefProp itself. Of course it is also possible to
      // define a pure fluid by specifying one component with a mole fraction equaling 1.

      // In the example below a mixture of two "refrigerants" R134a and butane is specified
      // to be the working fluid. The order of the mole fractions in array Conc must be
      // according the order of the component names in array Comp. Instead of specifying
      // the composition there also could have been chosen to specify the predefined mixture
      // e.g. R421a, which is mixture of 54% R125 and 46% R134a. A predefined mixture must be 
      // given as if it is a pure fluid:
      // strcpy (Comp[0], "R421a")
      // fluidprop_setfluid( "RefProp", 1, Comp, Conc)

      strcpy (Comp[0], "R134a"   );  Conc[0] = 0.46;
      strcpy (Comp[1], "butane"  );  Conc[1] = 0.54;
      fluidprop_setfluid( "RefProp", 2, Comp[0], LEN_COMPONENTS, Conc);

      if ( strcmp(fluidprop_geterror(), "No errors") )
         printf( "\n%s\n", fluidprop_geterror());
      else
      {
         printf( "\n" );
         printf( "RefProp, mixture of R134a and butane, P = 16 bar, T = 25 degr. C            \n" );
         printf( "----------------------------------------------------------------------------\n" );

         P = 16;
         T = 25;

         // To calculate some properties as function of two other independent properties
         // the appropriate function must be called, exactly like in other examples.
         // Next to various thermodynamic properties also properties related to CFD 
         // quantities and transport properties are available, as well as the liquid
         // and vapor phase compositions (not showed here). For clarity the writing
         // of the error message has been left out.

         printf( "Pressure      = %e%s\n" , fluidprop_pressure   ( "PT", P, T) , " bar      " );
         printf( "Temperature   = %e%s\n" , fluidprop_temperature( "PT", P, T) , " degr. C  " );
         printf( "Spec. volume  = %e%s\n" , fluidprop_specvolume ( "PT", P, T) , " m3/kg    " );
         printf( "Density       = %e%s\n" , fluidprop_density    ( "PT", P, T) , " kg/m3    " );
         printf( "Enthalpy      = %e%s\n" , fluidprop_enthalpy   ( "PT", P, T) , " kJ/kg    " );
         printf( "Entropy       = %e%s\n" , fluidprop_entropy    ( "PT", P, T) , " kJ/kg.K  " );
         printf( "Vapor qual.   = %e%s\n" , fluidprop_vaporqual  ( "PT", P, T) , " kg/kg    " );
         printf( "Isoc. Heat Cv = %e%s\n" , fluidprop_heatcapv   ( "PT", P, T) , " kJ/kg.K  " );
         printf( "Isob. Heat Cp = %e%s\n" , fluidprop_heatcapp   ( "PT", P, T) , " kJ/kg.K  " );
         printf( "Sound speed   = %e%s\n" , fluidprop_soundspeed ( "PT", P, T) , " m/s      " );
         printf( "Dyn. Viscos.  = %e%s\n" , fluidprop_viscosity  ( "PT", P, T) , " Pa.s     " );
         printf( "Th. Conduct.  = %e%s\n" , fluidprop_thermcond  ( "PT", P, T) , " W/m.K    " );

         // Note that an additional Add-on is required in order to compute the derivatives. 
         // A license for this Add-on can be purchased from the Asimptote web shop. If the 
         // appropriate license is not available on your system then an error message is 
         // displayed.             
         fluidprop_alpha( "PT", P, T);
         
         if ( strcmp(fluidprop_geterror(), "No errors") ) 
            printf( "\n%s\n", fluidprop_geterror());
         else 
         {
            printf( "Alpha         = %e%s\n" , fluidprop_alpha   ( "PT", P, T) , " m2/s2    " );
            printf( "Beta          = %e%s\n" , fluidprop_beta    ( "PT", P, T) , " kg/m3    " );
            printf( "Chi           = %e%s\n" , fluidprop_chi     ( "PT", P, T) , " Nm4/kg2.K" );
            printf( "Fi            = %e%s\n" , fluidprop_fi      ( "PT", P, T) , " 1/K      " );
            printf( "Ksi           = %e%s\n" , fluidprop_ksi     ( "PT", P, T) , " kg2/Nm4  " );
            printf( "Psi           = %e%s\n" , fluidprop_psi     ( "PT", P, T) , " s2/m2    " );
            printf( "Zeta          = %e%s\n" , fluidprop_zeta    ( "PT", P, T) , " kg.K/m3  " );
            printf( "Theta         = %e%s\n" , fluidprop_theta   ( "PT", P, T) , " 1/K      " );
            printf( "Kappa         = %e%s\n" , fluidprop_kappa   ( "PT", P, T) , " 1/bar    " );
            printf( "Gamma         = %e%s\n" , fluidprop_gamma   ( "PT", P, T) , " -        " );
         }
      }

      printf( "\n" );
      PressEnter();
   }

   //----------------------------------------------------------------------------------//
   //                            New features in FluidProp 2                           //
   //----------------------------------------------------------------------------------//

   // In the follwing examples some of the following new features of FluidProp 2 are
   // shown:
   //  1. Considerably improved performance
   //  2. Secondary tmd properties for two-phase region
   //  3. Method to calculate all properties available
   //  4. Methods returning Fluid info
   //  5. Unit conversion
   //  6. Name table with general names for all libraies
   //  7. GasMix renewed and extended
   //  8. Ability to use predefined mixtures in RefProp (see example above)

   if ( freeStanMix_Available)
   {
      // General names for all libraries are used now in both a long and
      // short format. FluidProp 1 names still can be used. So for hexane
      // could be entered the FP1 name n-Hexane as well as the FP2 general
      // names hexane and C6H14. In FP2 names or no longer case sensitive.
      strcpy (Comp[0], "c6h14");
      fluidprop_setfluid( "freeStanMix", 1, Comp[0], LEN_COMPONENTS, Conc);

      if ( strcmp(fluidprop_geterror(), "No errors") )
         printf( "\n%s\n", fluidprop_geterror());
      else
      {
         printf( "\n" );
         printf( "freeStanMix, Hexane, P = 1 bar, q = 0.5 (SetUnits and AllProps method)          \n" );
         printf( "----------------------------------------------------------------------------\n" );

         double P_in = 1;
         double q_in = 0.5;

         // In this example the use of the AllProps method is showed. This method calculates
         // all available properties in one go. Showing the feature of calculation the
         // secondary tmd properties in the 2-phase phase region the input specification
         // 'Pq' is used. At the same time unit conversions are shown by defining the
         // unit set to be SI units, with the exception of the pressure, temperature,
         // enthalpy and speed of sound successively to be in bar, degrees Fahrenheit,
         // Joule per mole and miles per hour.

         // Note that in this example the value of the isobaric heat capacity is not displayed
         // as it is not defined in the two-phase region for pure fluids. 
     
         // Again, note that an additional Add-on is required in order to compute the derivatives 
         // alpha, beta,..., gamma. Therefore the values of these properties are not presented
         // here. A license for this Add-on can be purchased from the Asimptote web shop. If 
         // the appropriate license is not available on your system then an error message is 
         // stored in the ErrorMsg variable.             

         fluidprop_setunits( "SI", "PerMass", "P,T,h,c", "bar,F,J/mol,mph");

         fluidprop_allprops( "Pq", P_in, q_in, &state, x, y);

         printf( "Pressure      = %e%s\n" , state.P     , " bar      " );
         printf( "Temperature   = %e%s\n" , state.T     , " degr.F   " );
         printf( "Density       = %e%s\n" , state.d     , " kg/m3    " );
         printf( "Spec. volume  = %e%s\n" , state.v     , " m3/kg    " );
         printf( "Enthalpy      = %e%s\n" , state.h     , " J/mol    " );
         printf( "Entropy       = %e%s\n" , state.s     , " J/kg.K   " );
         printf( "Vapor qual.   = %e%s\n" , state.q     , " kg/kg    " );
         printf( "Int. energy   = %e%s\n" , state.u     , " J/kg     " );
         printf( "Isoc. Heat Cv = %e%s\n" , state.cv    , " J/kg.K   " );
         printf( "Sound speed   = %e%s\n" , state.c     , " mph    \n" );

         // By specifying blanks the units will be default FluidProp units again.
         fluidprop_setunits( "", "", "", "");

         printf( "\n" );
         printf( "freeStanMix, Hexane, fluid info methods                                         \n" );
         printf( "----------------------------------------------------------------------------\n" );

         // Next there is an example of the use of the fluid info methods. Both the use
         // of methods returning a specific property and the method AllInfo returning
         // all info properties in one go are shown.

         double M_mol, T_crit, P_crit, T_min, T_max;
         fluidprop_allinfo( &M_mol, &T_crit, &P_crit, &T_min, &T_max);
         printf( "Molar mass    = %e%s%e%s\n" , M_mol   , " = " , fluidprop_mmol ( ) , " kg/mol   " );
         printf( "T critical    = %e%s%e%s\n" , T_crit  , " = " , fluidprop_tcrit( ) , " degr. C  " );
         printf( "P critical    = %e%s%e%s\n" , P_crit  , " = " , fluidprop_pcrit( ) , " bar      " );
         printf( "T min         = %e%s%e%s\n" , T_min   , " = " , fluidprop_tmin ( ) , " degr. C  " );
         printf( "T max         = %e%s%e%s\n" , T_max   , " = " , fluidprop_tmax ( ) , " degr. C  " );
      }

      printf( "\n" );
   }

   // At the end of the program you must free the fluidprop library by calling uninit_fluidprop()
   uninit_fluidprop();

   printf ("Done.\n");

   return 0;
}



