!============================================================================================!
!                                                                                            !
!                                    Intel Fortran Client                                    !
!                                    --------------------                                    !
!                                                                                            !
!  This is an example of a client application in Intel Fortran for FluidProp, a server       !
!  module for the calculation of thermophysical properties of fluids. FluidProp is a         !
!  common interface to GasMix, IF97, RefProp, (free)StanMix, TPSI, PCP-SAFT and is developed !
!  by Piero Colonna and Teus van der Stelt.                                                  !
!                                                                                            !
!  Teus van der Stelt                                                                        !
!  Asimptote BV                                                                              !
!                                                                                            !
!  July, 2004, for FluidProp 1                                                               !
!  January, 2006, for FluidProp 2                                                            !
!  April, 2007, for FluidProp 2.3                                                            !
!  May, 2009, for FluidProp 2.3                                                              !
!  November, 2018, for FluidProp 3.1 for macOS and Linux                                     !
!                                                                                            !
!============================================================================================!
      
   program Fortran_Sample_Client
      
      use FluidProp

      implicit none

      character(len=80)    :: ErrorMsg

      ! Number of components is generally not limited, but RefProp does not allow more
      ! than 20 components in a mixture and (free)StanMix does not allow more than 10.
      ! It is important however to allocate sufficient memory space.
      character(len=20)     :: Comp(20) = ' '
      real(8)               :: Conc(20) = 0.d0, x(20),y(20)

      integer(4)            :: nFluids, i, j, version(4)
      character(len=256)    :: FluidNames(1000)

      logical               :: TPSI_Available, freeStanMix_Available, IF97_Available,        &
                               GasMix_Available, RefProp_Available, StanMix_Available,       &
                               PCP_SAFT_Available
      
      real(8)               :: P_in, q_in, P, T, v, d, h, s, u, q, c, cv, cp, alpha_,        &
                               beta_, chi_, fi_, ksi_, psi_, zeta_, theta_, kappa_,          &
                               gamma_, eta, lambda
      real(8)               :: M_mol, T_crit, P_crit, T_min, T_max


!--------------------------------------------------------------------------------------------!

      ! Before you can use FluidProp you must create and initialize a FluidProp object 
      ! with the Init_FluidProp call.

      call Init_FluidProp()
     
      ! Before a property can be calculated, first a child object for the desired 
      ! thermophysical library must be created. This can be done by using the CreateObject 
      ! subroutine or by defining the working fluid directly with the SetFluid subroutine. 
      ! However subroutine SetFluid can be used for the same purpose too, with the 
      ! CreateObject subroutine you can easily create a list of available libraries in 
      ! your client.

      version = GetVersion("FluidProp");
      write(*,*)
      write(*,'( " Available libraries in FluidProp version ",3(I1,"."), I3)') version
      write(*,*) '----------------------------------------------------------------------------'
      
      ! Create an object for the TPSI library
      call CreateObject( 'TPSI', ErrorMsg)
      if ( ErrorMsg == 'No errors') then
         version = GetVersion("TPSI");
         write(*,'(" TPSI version          ",3(I1,"."), I2, " available")') version
         TPSI_Available = .true.
      else 
         write(*,*) 'Unable to create TPSI object: ',trim(ErrorMsg)
         TPSI_Available = .false.
      endif

      ! Create an object for the freeStanMix library
      call CreateObject( 'freeStanMix', ErrorMsg)
      if ( ErrorMsg == 'No errors') then
         version = GetVersion("freeStanMix");
         write(*,'(" freeStanMix version   ",3(I1,"."), I2, " available")') version
         freeStanMix_Available = .true.
      else 
         write(*,*) 'Unable to create freeStanMix object: ',trim(ErrorMsg)
         freeStanMix_Available = .false.
      endif

      ! Create an object for the IF97 library
      call CreateObject( 'IF97', ErrorMsg)
      if ( ErrorMsg == 'No errors') then
         version = GetVersion("IF97");
         write(*,'(" IF97 version          ",3(I1,"."), I2, " available")') version
         IF97_Available = .true.
      else 
         write(*,*) 'Unable to create IF97 object: ',trim(ErrorMsg)
         IF97_Available = .false.
      endif

      ! Create an object for the GasMix library
      call CreateObject( 'GasMix', ErrorMsg)
      if ( ErrorMsg == 'No errors') then
         version = GetVersion("GasMix");
         write(*,'(" GasMix version        ",3(I1,"."), I2, " available")') version
         GasMix_Available = .true.
      else 
         write(*,*) 'Unable to create GasMix object: ',trim(ErrorMsg)
         GasMix_Available = .false.
      endif

      ! Create an object for the RefProp library
      ! Note that an additional Add-on is required in order to run RefProp or, that, 
      ! if you are using MS-Windows, a version of REFPROP obtained from NIST has been 
      ! installed. A license for the Add-on can be purchased from the Asimptote web shop. 
      ! If the appropriate license is not available on your system then an error message 
      ! is displayed.
      call CreateObject( 'RefProp', ErrorMsg)
      if ( ErrorMsg == 'No errors') then
         version = GetVersion("RefProp");
         write(*,'(" RefProp version       ",3(I1,"."), I2, " available")') version
         RefProp_Available = .true.
      else 
         write(*,*) 'Unable to create RefProp object: ',trim(ErrorMsg)
         RefProp_Available = .false.
      endif

      ! Create an object for the StanMix library
      ! Note that an additional Add-on is required in order to run StanMix. A license for this 
      ! Add-on can be purchased from the Asimptote web shop. If the appropriate license is not 
      ! available on your system then an error message is displayed.
      call CreateObject( 'StanMix', ErrorMsg)
      if ( ErrorMsg == 'No errors') then
         version = GetVersion("StanMix");
         write(*,'(" StanMix version       ",3(I1,"."), I2, " available")') version
         StanMix_Available = .true.
      else
         write(*,*) 'Unable to create StanMix object: ',trim(ErrorMsg)
         StanMix_Available = .false.
      endif

      ! Create an object for the PCP-SAFT library
      ! Note that an additional Add-on is required in order to run PCP-SAFT. A license for this 
      ! Add-on can be purchased from the Asimptote web shop. If the appropriate license is not 
      ! available on your system then an error message is displayed.
      call CreateObject( 'PCP-SAFT', ErrorMsg)
      if ( ErrorMsg == 'No errors') then
         version = GetVersion("PCP-SAFT");
         write(*,'(" PCP-SAFT version      ",3(I1,"."), I2, " available")') version        
         PCP_SAFT_Available = .true.
      else 
         write(*,*) 'Unable to create PCP-SAFT object: ',trim(ErrorMsg)
         PCP_SAFT_Available = .false.
      endif

      write(*,*)
      pause

      !----------------------------------------------------------------------------------!
      !                                       TPSI                                       !
      !----------------------------------------------------------------------------------!

      if ( TPSI_Available) then
      
         ! After creation of the object of the desired library, first the working fluid 
         ! must be defined before a property can be calculated. In this example methane
         ! from the TPSI library is defined as the working fluid. For TPSI there is no
         ! need to specify concentation Conc, because there is always just 1 component.
     
         Comp(1) = 'Methane'
         call SetFluid( 'TPSI', 1, Comp, Conc, ErrorMsg)

         if ( ErrorMsg /= 'No errors') then

            write(*,*)
            write(*,*) ErrorMsg
         
         else
         
            write(*,*)
            write(*,*) 'TPSI, Methane, P = 1 bar, T = 15 degr.C'
            write(*,*) '----------------------------------------------------------------------------'
 
            P = 1.d0
            T = 15.d0

            ! To calculate some properties as function of two other independent properties
            ! the appropriate function must be called. In the example below the density,
            ! specific volume, enthalpy, entropy and vapor quality are calculated as 
            ! function of the pressure and temperature. There is a check if the pressure  
            ! and temperature function return the same value as entered. 
     
            write(*,*) 'Pressure      = ', Pressure   ( 'PT', P, T, ErrorMsg), ' bar    '
            if ( ErrorMsg /= 'No errors') write(*,*) ErrorMsg
            write(*,*) 'Temperature   = ', Temperature( 'PT', P, T, ErrorMsg), ' degr. C'
            if ( ErrorMsg /= 'No errors') write(*,*) ErrorMsg
            write(*,*) 'Density       = ', Density    ( 'PT', P, T, ErrorMsg), ' kg/m3  '
            if ( ErrorMsg /= 'No errors') write(*,*) ErrorMsg
            write(*,*) 'Spec. volume  = ', SpecVolume ( 'PT', P, T, ErrorMsg), ' m3/kg  '
            if ( ErrorMsg /= 'No errors') write(*,*) ErrorMsg
            write(*,*) 'Enthalpy      = ', Enthalpy   ( 'PT', P, T, ErrorMsg), ' kJ/kg  '
            if ( ErrorMsg /= 'No errors') write(*,*) ErrorMsg
            write(*,*) 'Entropy       = ', Entropy    ( 'PT', P, T, ErrorMsg), ' kJ/kg.K'
            if ( ErrorMsg /= 'No errors') write(*,*) ErrorMsg
            write(*,*) 'Vapor quality = ', VaporQual  ( 'PT', P, T, ErrorMsg), ' kg/kg  '
            if ( ErrorMsg /= 'No errors') write(*,*) ErrorMsg
            write(*,*) 

            ! You can retrieve a list of available fluids in a library by calling 
            ! subroutine GetFluidNames. The number of fluids and an array with 
            ! fluidnames is returned. The number of fluids displayed depends on the 
            ! license and not all fluids are available for each sub-library. For the 
            ! free version the number of fluids is most limited. 
     
            call GetFluidNames( 'long', 'TPSI', nFluids, FluidNames, ErrorMsg)

            write(*,*)
            write(*,*) 'Available components in TPSI'
            write(*,*) '----------------------------------------------------------------------------'

            if ( ErrorMsg == 'No errors') then
               write(*,*) 'Number of available fluids = ', nFluids
               write(*,*) 'The available fluids are: ' 
               write(*,*)
               do i = 1, nFluids, 3
                  write(*,'(2x,3a22)') (FluidNames(j), j = i, i + 2)
               enddo
            endif
         endif
   
         write(*,*)
         pause

      endif
   
      ! For the other libraries the method is essentially the same. The only 
      ! difference is in the way arguments are supplied to the SetFluid subroutine. 
      ! Furthermore the properties that can be calculated varies for each library, 
      ! as well as the two the independent properties that can be used as input
      ! (see user documentation).     
      ! In the examples below for each library the use of subroutine SetFluid is 
      ! explained in detail, together with the calculation of several properties.
     
      !----------------------------------------------------------------------------------!
      !                                   freeStanMix                                    !
      !----------------------------------------------------------------------------------!

      if ( freeStanMix_Available) then
      
         ! You can retrieve a list of available fluids in a library by calling 
         ! subroutine GetFluidNames. The number of fluids and an array with
         ! fluidnames is returned. 
     
         FluidNames(:) = ' '
         call GetFluidNames( 'long', 'freeStanMix', nFluids, FluidNames, ErrorMsg)

         write(*,*)
         write(*,*) 'Available components in freeStanMix'
         write(*,*) '----------------------------------------------------------------------------'

         if ( ErrorMsg == 'No errors') then
            write(*,*) 'Number of available fluids = ', nFluids
            write(*,*) 'The available fluids are: ' 
            write(*,*)
            do i = 1, nFluids, 3
               write(*,'(2x,3a22)') (FluidNames(j), j = i, i + 2)
            enddo
         endif

         write(*,*)
         pause

         ! For freeStanMix you have three possiblities: you can use pure fluids, predefined mixtures,
         ! or you can use mixtures and assign different mole fractions to concentrations of
         ! of the components. 

         ! In the first example a pure fluid is selected. There is no need to specify a 
         ! concentration, it is sufficient to specify the fluidname, in this case ethanol, 
         ! and the number of components being 1.

         Comp(1) = 'ethanol'
         call SetFluid( 'freeStanMix', 1, Comp, Conc, ErrorMsg)

         if ( ErrorMsg /= 'No errors') then

            write(*,*)
            write(*,*) ErrorMsg
         
         else
         
            write(*,*)
            write(*,*) 'freeStanMix, ethanol, P = 2 bar, T = 120 degr.C'
            write(*,*) '----------------------------------------------------------------------------'
 
            P = 2.d0
            T = 120.d0

            ! To calculate some properties as function of two other independent properties
            ! the appropriate function must be called. In the example a large number of 
            ! properties is calculated. Next to various thermodynamic properties also 
            ! properties related to CFD quantities are available. For clarity the writing
            ! of the error message has been left out.
     
            write(*,*) 'Pressure      = ', Pressure   ( 'PT', P, T, ErrorMsg), ' bar    '
            write(*,*) 'Temperature   = ', Temperature( 'PT', P, T, ErrorMsg), ' degr. C'
            write(*,*) 'Density       = ', Density    ( 'PT', P, T, ErrorMsg), ' kg/m3  '
            write(*,*) 'Spec. volume  = ', SpecVolume ( 'PT', P, T, ErrorMsg), ' m3/kg  '
            write(*,*) 'Enthalpy      = ', Enthalpy   ( 'PT', P, T, ErrorMsg), ' kJ/kg  '
            write(*,*) 'Entropy       = ', Entropy    ( 'PT', P, T, ErrorMsg), ' kJ/kg.K'
            write(*,*) 'Vapor quality = ', VaporQual  ( 'PT', P, T, ErrorMsg), ' kg/kg  '
            write(*,*) 'Int. energy   = ', IntEnergy  ( 'PT', P, T, ErrorMsg), ' kJ/kg    '
            write(*,*) 'Isoc. Heat Cv = ', HeatCapV   ( 'PT', P, T, ErrorMsg), ' kJ/kg.K  '
            write(*,*) 'Isob. Heat Cp = ', HeatCapP   ( 'PT', P, T, ErrorMsg), ' kJ/kg.K  '
            write(*,*) 'Sound speed   = ', SoundSpeed ( 'PT', P, T, ErrorMsg), ' m/s      '
            
			! Note that an additional Add-on is required in order to compute the derivatives. 
			! A license for this Add-on can be purchased from the Asimptote web shop. If the 
			! appropriate license is not available on your system then an error message is 
			! displayed.             
            alpha_ = Alpha( 'PT', P, T, ErrorMsg)

            if ( ErrorMsg /= 'No errors') then 
               write(*,*) 
               write(*,*) ErrorMsg
            else
               write(*,*) 'Alpha         = ', Alpha      ( 'PT', P, T, ErrorMsg), ' m2/s2    '
               write(*,*) 'Beta          = ', Beta       ( 'PT', P, T, ErrorMsg), ' kg/m3    '
               write(*,*) 'Chi           = ', Chi        ( 'PT', P, T, ErrorMsg), ' Nm4/kg2.K'
               write(*,*) 'Fi            = ', Fi         ( 'PT', P, T, ErrorMsg), ' 1/K      '
               write(*,*) 'Ksi           = ', Ksi        ( 'PT', P, T, ErrorMsg), ' kg2/N.m4 '
               write(*,*) 'Psi           = ', Psi        ( 'PT', P, T, ErrorMsg), ' s2/m2    '
               write(*,*) 'Zeta          = ', Zeta       ( 'PT', P, T, ErrorMsg), ' kg.K/m3  '
               write(*,*) 'Theta         = ', Theta      ( 'PT', P, T, ErrorMsg), ' 1/K      '
               write(*,*) 'Kappa         = ', Kappa      ( 'PT', P, T, ErrorMsg), ' 1/bar    '
               write(*,*) 'Gamma         = ', Gamma      ( 'PT', P, T, ErrorMsg), ' -        '
            end if
            write(*,*) 

         endif
      
         write(*,*)     
         pause
 
         ! In the second example a predefined mixture is selected. Here is also no need 
         ! to specify a concentration, because it has been stored in the mixture input file
         ! file. It is sufficient to specify the mixture name, in this case propane/pentane
         ! (mixture of 50% mole percent propane and 50% mole percent pentane). For the use
         ! of the mixture with the stored concentrations, the number of components must be 1.

         Comp(1) = 'propane/pentane';
         call SetFluid( 'freeStanMix', 1, Comp, Conc, ErrorMsg)

         if ( ErrorMsg /= 'No errors') then

            write(*,*)
            write(*,*) ErrorMsg
         
         else
         
            write(*,*)
            write(*,*) 'freeStanMix, propane/pentane (predef. mix: 50%-50%), P = 5 bar, T = 50 degr.C'
            write(*,*) '------------------------------------------------------------------------------'

            P = 5.d0
            T = 50.d0

            ! To calculate some properties as function of two other independent properties
            ! the appropriate function must be called. In this example additionally the 
            ! liquid and vapor phase composition are calculated. For clarity the writing
            ! of the error message has been left out.
     
            write(*,*) 'Pressure      = ', Pressure   ( 'PT', P, T, ErrorMsg), '  bar    '
            write(*,*) 'Temperature   = ', Temperature( 'PT', P, T, ErrorMsg), '  degr.C '
            write(*,*) 'Spec. volume  = ', SpecVolume ( 'PT', P, T, ErrorMsg), '  m3/kg  '
            write(*,*) 'Density       = ', Density    ( 'PT', P, T, ErrorMsg), '  kg/m3  '
            write(*,*) 'Enthalpy      = ', Enthalpy   ( 'PT', P, T, ErrorMsg), '  kJ/kg  '
            write(*,*) 'Entropy       = ', Entropy    ( 'PT', P, T, ErrorMsg), '  kJ/kg.K'
            write(*,*) 'Vapor qual.   = ', VaporQual  ( 'PT', P, T, ErrorMsg), '  kg/kg  '
            write(*,*)

            call LiquidCmp('PT', P, T, x, ErrorMsg) 
            write(*,*) 'Liq phase cmp = propane : ', x(1), ' mole frac '
            write(*,*) '                pentane : ', x(2), ' mole frac '
            write(*,*)
         
            call VaporCmp ('PT', P, T, y, ErrorMsg)       
            write(*,*) 'Vap phase cmp = propane : ', y(1), ' mole frac '
            write(*,*) '                pentane : ', y(2), ' mole frac '
            write(*,*)
         endif

         write(*,*)     
         pause
 
         ! In the third freeStanMix example the same mixture is selected as in the second 
         ! example, but now the mole fractions are adapted. To do this you must specify
         ! the real number of components in the mixture, the mixture name and concentrations. 
         ! In most of the available mixtures the first component comes first in the mixture 
         ! name and its concentration must be stored in the first element of the concentration 
         ! array; the concentration of the second component must be stored in the second 
         ! element, etc. So, concentrations must be stored in the order as the component names
         ! appear in the mixture name. 

         Comp(1) = 'propane/pentane';
         Conc(1) = 0.75d0
         Conc(2) = 1.d0 - Conc(1)
         call SetFluid( 'freeStanMix', 2, Comp, Conc, ErrorMsg)

         if ( ErrorMsg /= 'No errors') then

            write(*,*)
            write(*,*) ErrorMsg
         
         else
         
            write(*,*)
            write(*,*) 'freeStanMix, propane/pentane (mix. 75%-25%), P = 5 bar, T = 40 degr.C'
            write(*,*) '-----------------------------------------------------------------------------'

            P = 5.d0
            T = 40.d0

            ! To calculate some properties as function of two other independent properties
            ! the appropriate function must be called. In this example the same properties
            ! are calculated as in the example above.
     
            write(*,*) 'Pressure      = ', Pressure   ( 'PT', P, T, ErrorMsg), '  bar    '
            write(*,*) 'Temperature   = ', Temperature( 'PT', P, T, ErrorMsg), '  degr.C '
            write(*,*) 'Spec. volume  = ', SpecVolume ( 'PT', P, T, ErrorMsg), '  m3/kg  '
            write(*,*) 'Density       = ', Density    ( 'PT', P, T, ErrorMsg), '  kg/m3  '
            write(*,*) 'Enthalpy      = ', Enthalpy   ( 'PT', P, T, ErrorMsg), '  kJ/kg  '
            write(*,*) 'Entropy       = ', Entropy    ( 'PT', P, T, ErrorMsg), '  kJ/kg.K'
            write(*,*) 'Vapor qual.   = ', VaporQual  ( 'PT', P, T, ErrorMsg), '  kg/kg  '
            write(*,*)

            call LiquidCmp('PT', P, T, x, ErrorMsg) 
            write(*,*) 'Liq phase cmp = propane : ', x(1), ' mole frac '
            write(*,*) '                pentane : ', x(2), ' mole frac '
            write(*,*)
         
            call VaporCmp ('PT', P, T, y, ErrorMsg)       
            write(*,*) 'Vap phase cmp = propane : ', y(1), ' mole frac '
            write(*,*) '                pentane : ', y(2), ' mole frac '
            write(*,*)
         endif
   
         write(*,*)     
         pause
 
      endif
       
      !----------------------------------------------------------------------------------!
      !                               IF97 water and steam                               !
      !----------------------------------------------------------------------------------!

      if ( IF97_Available) then
      
         ! IF97 just contains a model for water and steam, so for IF97 the number of 
         ! components, component name(s) and concentrations do not matter. It is 
         ! sufficient to specify the library name 'IF97' and to declare the arrays 
         ! Comp and Conc as a character array and a real(8) array in the way it is 
         ! done in the top of this sample program. So, there is also no need to retrieve
         ! a list of available fluids by calling subroutine GetFluidNames.    

         ! Re-init concentrations and component names. This is not really necessary but
         ! it shows the independence for IF97.
         Conc(:) = 0.d0
         Comp(:) = ' '
         call SetFluid( 'IF97', 1, Comp, Conc, ErrorMsg)
      
         if ( ErrorMsg /= 'No errors') then

            write(*,*)
            write(*,*) ErrorMsg
         
         else
         
            write(*,*)
            write(*,*) 'IF97 water/steam, P = 40 bar, h = 3000 kJ/kg'
            write(*,*) '----------------------------------------------------------------------------'

            P = 40.d0
            h = 3000.d0

            ! To calculate some properties as function of two other independent properties
            ! the appropriate function must be called, exactly like in the other examples. 
            ! In this example all available functions for IF97 are called and for the input 
            ! the pressure and enthalpy are entered. For clarity the writing of the error 
            ! message has been left out.
     
            write(*,*) 'Pressure      = ', Pressure   ( 'Ph', P, h, ErrorMsg), ' bar    '
            write(*,*) 'Temperature   = ', Temperature( 'Ph', P, h, ErrorMsg), ' degr.C '
            write(*,*) 'Spec. volume  = ', SpecVolume ( 'Ph', P, h, ErrorMsg), ' m3/kg  '
            write(*,*) 'Density       = ', Density    ( 'Ph', P, h, ErrorMsg), ' kg/m3  '
            write(*,*) 'Enthalpy      = ', Enthalpy   ( 'Ph', P, h, ErrorMsg), ' kJ/kg  '
            write(*,*) 'Entropy       = ', Entropy    ( 'Ph', P, h, ErrorMsg), ' kJ/kg.K'
            write(*,*) 'Int. Energy   = ', IntEnergy  ( 'Ph', P, h, ErrorMsg), ' kJ/kg  '
            write(*,*) 'Vapor qual.   = ', VaporQual  ( 'Ph', P, h, ErrorMsg), ' kg/kg  '
            write(*,*) 'Isoc. Heat Cv = ', HeatCapV   ( 'Ph', P, h, ErrorMsg), ' kJ/kg.K'
            write(*,*) 'Isob. Heat Cp = ', HeatCapP   ( 'Ph', P, h, ErrorMsg), ' kJ/kg.K'
            write(*,*) 'Sound speed   = ', SoundSpeed ( 'Ph', P, h, ErrorMsg), ' m/s    '
            write(*,*) 'Dyn. Viscos.  = ', Viscosity  ( 'Ph', P, h, ErrorMsg), ' Pa.s   '
            write(*,*) 'Th. Conduct.  = ', ThermCond  ( 'Ph', P, h, ErrorMsg), ' W/m.K  '
            write(*,*) 

         endif

         write(*,*)
         pause

      endif
      
      !----------------------------------------------------------------------------------!
      !                                     GasMix                                       !
      !----------------------------------------------------------------------------------!

      if ( GasMix_Available) then
      
         ! Using GasMix you can calculate properties of ideal gases and mixtures of ideal
         ! gasses. You can retrieve a list of available components by calling subroutine 
         ! GetFluidNames. The number of fluids and an array with fluidnames are returned. 

         FluidNames(:) = ' '
         call GetFluidNames( 'short', 'GasMix', nFluids, FluidNames, ErrorMsg)

         write(*,*)
         write(*,*) 'Available components in GasMix'
         write(*,*) '----------------------------------------------------------------------------'

         if ( ErrorMsg == 'No errors') then
            write(*,*) 'Number of available fluids = ', nFluids
            write(*,*) 'The available fluids are: ' 
            write(*,*)
            do i = 1, nFluids, 3
               write(*,'(2x,3a22)') (FluidNames(j), j = i, i + 2)
            enddo
         endif

         write(*,*)
         pause

         ! For GasMix there is just one possibility to define the working fluid. You must 
         ! specify the composition by specifying successively the number of components, the
         ! components and the molefractions. At the moment there is no feature for predefined 
         ! mixtures.

         ! In the example below an air compostion is specified to be the working fluid. The 
         ! order of the mole fractions in array Conc must be according the order of the 
         ! component names in array Comp. It is also possible to define a pure fluid by 
         ! specifying one component with a mole fraction equaling 1.

         Comp(1) = 'N2'   ;  Conc(1) = 0.7801d0
         Comp(2) = 'O2'   ;  Conc(2) = 0.2094d0
         Comp(3) = 'H2O'  ;  Conc(3) = 0.0102d0
         Comp(4) = 'CO2'  ;  Conc(4) = 0.0003d0
         call SetFluid( 'GasMix', 4, Comp, Conc, ErrorMsg)

         if ( ErrorMsg /= 'No errors') then

            write(*,*)
            write(*,*) ErrorMsg
         
         else
         
            write(*,*)
            write(*,*) 'GasMix, Air, P = 1 bar, T = 15 degr.C'
            write(*,*) '----------------------------------------------------------------------------'

            P = 1.d0
            T = 15.d0

            ! To calculate some properties as function of two other independent properties
            ! the appropriate function must be called, exactly like in other examples. For 
            ! clarity the writing of the error message has been left out.
     
            h = Enthalpy( 'PT', P, T, ErrorMsg)
            s = Entropy ( 'Ph', P, h, ErrorMsg)
            write(*,*) 'Enthalpy      = ', h                                 , ' kJ/kg  '
            write(*,*) 'Entropy       = ', s                                 , ' kJ/kg.K'
            write(*,*) 'Pressure      = ', Pressure   ( 'Ps', P, s, ErrorMsg), ' bar    '
            write(*,*) 'Temperature   = ', Temperature( 'Ph', P, h, ErrorMsg), ' degr.C '           
            write(*,*) 'Spec. volume  = ', SpecVolume ( 'Ps', P, s, ErrorMsg), ' m3/kg  '
            write(*,*) 'Density       = ', Density    ( 'Ph', P, h, ErrorMsg), ' kg/m3  '
            write(*,*) 'Isob. Heat Cp = ', HeatCapP   ( 'PT', P, T, ErrorMsg), ' kJ/kg.K'
            write(*,*) 'Dyn. Viscos.  = ', Viscosity  ( 'PT', P, T, ErrorMsg), ' Pa.s   '
            write(*,*) 'Th. Conduct.  = ', ThermCond  ( 'PT', P, T, ErrorMsg), ' W/m.K  '

         endif

         write(*,*)
         pause

      endif

      !----------------------------------------------------------------------------------!
      !                                     RefProp                                      !
      !----------------------------------------------------------------------------------!

      if ( RefProp_Available) then
      
         ! Using RefProp you can calculate properties of pure fluids and mixtures of 
         ! fluids. You can retrieve a list of available components by calling subroutine 
         ! GetFluidNames. The number of fluids and an array with fluidnames is returned. 
         
         ! Note that an additional Add-on is required in order to run RefProp or, that, 
         ! if you are using MS-Windows, a version of REFPROP obtained from NIST has been 
         ! installed. A license for the Add-on can be purchased from the Asimptote web shop. 
         ! If the appropriate license is not available on your system then an error message 
         ! is displayed.

         FluidNames(:) = ' '
         call GetFluidNames( 'long', 'RefProp', nFluids, FluidNames, ErrorMsg)

         write(*,*)
         write(*,*) 'Available components in RefProp'
         write(*,*) '----------------------------------------------------------------------------'
 
         if ( ErrorMsg == 'No errors') then
            write(*,*) 'Number of available fluids = ', nFluids
            write(*,*) 'The fluids are: ' 
            write(*,*)
            do i = 1, nFluids, 3
               write(*,'(2x,3a22)') (FluidNames(j), j = i, i + 2)
            enddo
         endif

         write(*,*)
         pause

         ! In RefProp there are two ways to define the working fluid. First you can specify 
         ! the composition by specifying successively the number of components, the component 
         ! names and the mole fractions, like in GasMix. Second you can define predefined 
         ! mixtures that are available in RefProp itself. Of course it is also possible to 
         ! define a pure fluid by specifying one component with a mole fraction equaling 1.

         ! In the example below a mixture of two "refrigerants" R134a and butane is specified 
         ! to be the working fluid. The order of the mole fractions in array Conc must be 
         ! according the order of the component names in array Comp. Instead of specifying
         ! the composition there also could have been chosen to specify a predefined mixture,
         ! e.g. R421a, which is mixture of 54% R125 and 46% R134a. A predefined mixture must 
         ! be given as if it is a pure fluid:
         ! Comp(1) = 'R421a'
         ! call SetFluid( 'RefProp', 1, Comp, Conc, ErrorMsg)

         Comp(1) = 'R134a'  ;  Conc(1) = 0.46d0
         Comp(2) = 'butane' ;  Conc(2) = 0.54d0
         call SetFluid( 'RefProp', 2, Comp, Conc, ErrorMsg)

         if ( ErrorMsg /= 'No errors') then

            write(*,*)
            write(*,*) ErrorMsg
         
         else
         
            write(*,*)
            write(*,*) 'RefProp, mixture of R134a and butane, P = 16 bar, T = 25 degr. C'
            write(*,*) '----------------------------------------------------------------------------'

            P = 16.d0
            T = 25.d0

            ! To calculate some properties as function of two other independent properties
            ! the appropriate function must be called, exactly like in other examples. 
            ! Next to various thermodynamic properties also properties related to CFD 
            ! quantities and transport properties are available, as well as the liquid
            ! and vapor phase compositions (not showed here). For clarity the writing 
            ! of the error message has been left out.

            write(*,*) 'Pressure      = ', Pressure   ( 'PT', P, T, ErrorMsg), ' bar      '
            write(*,*) 'Temperature   = ', Temperature( 'PT', P, T, ErrorMsg), ' degr. C  '
            write(*,*) 'Spec. volume  = ', SpecVolume ( 'PT', P, T, ErrorMsg), ' m3/kg    '
            write(*,*) 'Density       = ', Density    ( 'PT', P, T, ErrorMsg), ' kg/m3    '
            write(*,*) 'Enthalpy      = ', Enthalpy   ( 'PT', P, T, ErrorMsg), ' kJ/kg    '
            write(*,*) 'Entropy       = ', Entropy    ( 'PT', P, T, ErrorMsg), ' kJ/kg.K  '
            write(*,*) 'Vapor qual.   = ', VaporQual  ( 'PT', P, T, ErrorMsg), ' kg/kg    '
            write(*,*) 'Isoc. Heat Cv = ', HeatCapV   ( 'PT', P, T, ErrorMsg), ' kJ/kg.K  '
            write(*,*) 'Isob. Heat Cp = ', HeatCapP   ( 'PT', P, T, ErrorMsg), ' kJ/kg.K  '
            write(*,*) 'Sound speed   = ', SoundSpeed ( 'PT', P, T, ErrorMsg), ' m/s      '
            write(*,*) 'Dyn. Viscos.  = ', Viscosity  ( 'PT', P, T, ErrorMsg), ' Pa.s     '
            write(*,*) 'Th. Conduct.  = ', ThermCond  ( 'PT', P, T, ErrorMsg), ' W/m.K    '

			! Note that an additional Add-on is required in order to compute the derivatives. 
			! A license for this Add-on can be purchased from the Asimptote web shop. If the 
			! appropriate license is not available on your system then an error message is 
			! displayed.             
            alpha_ = Alpha( 'PT', P, T, ErrorMsg)

            if ( ErrorMsg /= 'No errors') then 
               write(*,*) 
               write(*,*) ErrorMsg
            else
               write(*,*) 'Alpha         = ', Alpha      ( 'PT', P, T, ErrorMsg), ' m2/s2    '
               write(*,*) 'Beta          = ', Beta       ( 'PT', P, T, ErrorMsg), ' kg/m3    '
               write(*,*) 'Chi           = ', Chi        ( 'PT', P, T, ErrorMsg), ' Nm4/kg2.K'
               write(*,*) 'Fi            = ', Fi         ( 'PT', P, T, ErrorMsg), ' 1/K      '
               write(*,*) 'Ksi           = ', Ksi        ( 'PT', P, T, ErrorMsg), ' kg2/N.m4 '
               write(*,*) 'Psi           = ', Psi        ( 'PT', P, T, ErrorMsg), ' s2/m2    '
               write(*,*) 'Zeta          = ', Zeta       ( 'PT', P, T, ErrorMsg), ' kg.K/m3  '
               write(*,*) 'Theta         = ', Theta      ( 'PT', P, T, ErrorMsg), ' 1/K      '
               write(*,*) 'Kappa         = ', Kappa      ( 'PT', P, T, ErrorMsg), ' 1/bar    '
               write(*,*) 'Gamma         = ', Gamma      ( 'PT', P, T, ErrorMsg), ' -        '
            end if
            write(*,*) 

         endif

         write(*,*)
         pause

      endif

      !----------------------------------------------------------------------------------!
      !                            New features in FluidProp 2                           !
      !----------------------------------------------------------------------------------!

      ! In the follwing examples some of the following new features of FluidProp 2 are 
      ! shown:
      !  1. Considerably improved performance
      !  2. Secondary tmd properties for two-phase region
      !  3. Method to calculate all properties available
      !  4. Methods returning Fluid info 
      !  5. Unit conversion
      !  6. Name table with general names for all libraies
      !  7. GasMix renewed and extended
      !  8. Ability to use predefined mixtures in RefProp (see example above)

      if ( freeStanMix_Available) then
      
         ! General names for all libraries are used now in both a long and
         ! short format. FluidProp 1 names still can be used. So for hexane
         ! could be entered the FP1 name n-Hexane as well as the FP2 general 
         ! names hexane and C6H14. In FP2 names or no longer case sensitive.
         Comp(1) = 'c6h14' 
         call SetFluid( 'freeStanMix', 1, Comp, Conc, ErrorMsg)

         if ( ErrorMsg /= 'No errors') then

            write(*,*)
            write(*,*) ErrorMsg
         
         else
         
            write(*,*)
            write(*,*) 'freeStanMix, hexane, P = 1 bar, q = 0.5 (SetUnits and AllProps method)'
            write(*,*) '----------------------------------------------------------------------------'
 
            P_in = 1.d0
            q_in = 0.5d0

            ! In this example the use of the AllProps method is showed. This method calculates
            ! all available properties in one go. Showing the feature of calculation the 
            ! secondary tmd properties in the 2-phase phase region the input specification
            ! 'Pq' is used. At the same time unit conversions are shown by defining the
            ! unit set to be SI units, with the exception of the pressure, temperature,
            ! enthalpy and speed of sound successively to be in bar, degrees Fahrenheit, 
            ! Joule per mole and miles per hour. 
            
            ! Note that in this example the value of the isobaric heat capacity is not displayed
            ! as it is not defined in the two-phase region for pure fluids. 
     
			! Again note that an additional Add-on is required in order to compute the derivatives 
			! alpha_, beta_,..., gamma_. Therefore the values of these properties are not presented
			! here. A license for this Add-on can be purchased from the Asimptote web shop. If 
			! the appropriate license is not available on your system then an error message is 
			! stored in the ErrorMsg variable.             

            call SetUnits( 'SI', 'PerMass', 'P,T,h,c', 'bar,F,J/mol,mph', ErrorMsg)
 
            call AllProps( 'Pq', P_in, q_in, P, T, v, d, h, s, u, q, x, y, cv, cp, c,        &
                            alpha_, beta_, chi_, fi_, ksi_, psi_, zeta_, theta_, kappa_ ,    &
                            gamma_, eta, lambda, ErrorMsg)

            write(*,*) 'Pressure      = ', P     , ' bar      '
            write(*,*) 'Temperature   = ', T     , ' degr.F   '
            write(*,*) 'Density       = ', d     , ' kg/m3    '
            write(*,*) 'Spec. volume  = ', v     , ' m3/kg    '
            write(*,*) 'Enthalpy      = ', h     , ' J/mol    '
            write(*,*) 'Entropy       = ', s     , ' J/kg.K   '
            write(*,*) 'Vapor quality = ', q     , ' kg/kg    '
            write(*,*) 'Int. energy   = ', u     , ' J/kg     '
            write(*,*) 'Isoc. Heat Cv = ', cv    , ' J/kg.K  '
            write(*,*) 'Sound speed   = ', c     , ' mph      '
            write(*,*)

            ! By specifying blanks the units will be default FluidProp units again.
            call SetUnits( '', '', '', '', ErrorMsg)
 
            write(*,*)
            write(*,*) 'freeStanMix, hexane, fluid info methods'
            write(*,*) '----------------------------------------------------------------------------'
 
            ! Next there is an example of the use of the fluid info methods. Both the use
            ! of methods returning a specific property and the method AllInfo returning
            ! all info properties in one go are shown.
     
            call AllInfo( M_mol, T_crit, P_crit, T_min, T_max, ErrorMsg);
            write(*,*) 'Molar mass    = ', M_mol , ' = ',Mmol ( ErrorMsg), ' kg/mol   '
            write(*,*) 'T critical    = ', T_crit, ' = ',Tcrit( ErrorMsg), ' degr.C   '
            write(*,*) 'P critical    = ', P_crit, ' = ',Pcrit( ErrorMsg), ' bar      '
            write(*,*) 'T min         = ', T_min , ' = ',Tmin ( ErrorMsg), ' degr.C   '
            write(*,*) 'T max         = ', T_max , ' = ',Tmax ( ErrorMsg), ' degr.C   '
            write(*,*)

         endif
      
         write(*,*)     
 
      endif 

      ! At the end of program, you should clean up your memory by deleting
      ! the FluidProp objects created in the beginning of the program with 
      ! the Init_FluidProp call and the FluidProp child objects created with  
      ! the SetFluid or CreateObject calls.

      call ReleaseObjects()

   end program Fortran_Sample_Client

!===================================================================================== EOF ==!
